@@ -174,13 +174,13 @@ class IManager(ABC):
         raise NotImplementedError("You must implement `connect_tty` method.")
 
     @abstractmethod
-    def exec(self, machine_name: str, command: str, lab_hash: Optional[str] = None, lab_name: Optional[str] = None) -> \
-            Generator[Tuple[bytes, bytes], None, None]:
+    def exec(self, machine_name: str, command: List[str], lab_hash: Optional[str] = None,
+             lab_name: Optional[str] = None) -> Generator[Tuple[bytes, bytes], None, None]:
         """Exec a command on a device in a running network scenario.
 
         Args:
             machine_name (str): The name of the device to connect.
-            command (str): The command to exec on the device.
+            command (List[str]): The command to exec on the device.
             lab_hash (Optional[str]): The hash of the network scenario where the device is deployed.
             lab_name (Optional[str]): The name of the network scenario where the device is deployed.
 
