@@ -17,31 +17,19 @@ from ... import utils
 from ...decorators import privileged
 from ...exceptions import DockerDaemonConnectionError, LinkNotFoundError, MachineCollisionDomainError, \
     InvocationError, LabNotFoundError
+from ...exceptions import MachineNotFoundError
 from ...foundation.manager.IManager import IManager
 from ...model.Lab import Lab
 from ...model.Link import Link
 from ...model.Machine import Machine
 from ...setting.Setting import Setting
-from ...utils import pack_files_for_tar
-from ...exceptions import MachineNotFoundError
+from ...utils import pack_files_for_tar, import_pywintypes
 
-
-def pywin_import_stub():
-    """Stub module of pywintypes for Unix systems (so it won't raise any `module not found` exception)."""
-    import types
-    pywintypes = types.ModuleType("pywintypes")
-    pywintypes.error = RequestsConnectionError
-    return pywintypes
-
-
-def pywin_import_win():
-    import pywintypes
-    return pywintypes
+pywintypes = import_pywintypes()
 
 
 def check_docker_status(method):
     """Decorator function to check if Docker daemon is running properly."""
-    pywintypes = utils.exec_by_platform(pywin_import_stub, pywin_import_win, pywin_import_stub)
 
     @privileged
     def check_docker(*args, **kw):
@@ -335,13 +323,13 @@ class DockerManager(IManager):
                                     )
 
     @privileged
-    def exec(self, machine_name: str, command: str, lab_hash: Optional[str] = None, lab_name: Optional[str] = None) -> \
-            Generator[Tuple[bytes, bytes], None, None]:
+    def exec(self, machine_name: str, command: List[str], lab_hash: Optional[str] = None,
+             lab_name: Optional[str] = None) -> Generator[Tuple[bytes, bytes], None, None]:
         """Exec a command on a device in a running network scenario.
 
         Args:
             machine_name (str): The name of the device to connect.
-            command (str): The command to exec on the device.
+            command (List[str]): The command to exec on the device.
             lab_hash (Optional[str]): The hash of the network scenario where the device is deployed.
             lab_name (Optional[str]): The name of the network scenario where the device is deployed.
 
